// ==============================================================
//                 ORBITER MODULE: LASERTANK1
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// LASERTANK1.cpp
// Control module for LASERTANK1 vessel class
//

// ==============================================================
#define ORBITER_MODULE
#define ENG 				1	
#define START 				2	
#define STOP 				3
#define CANNON 				4
#include "orbitersdk.h"
#include "LASERTANK1.h"
#include "OrbiterSoundSDK35.h"
VISHANDLE MainExternalMeshVisual = 0;



void LASERTANK1::clbkPostCreation (void)

{
JohnSoundID=ConnectToOrbiterSoundDLL3(GetHandle());
RequestLoadVesselWave3(JohnSoundID,ENG,"Sound\\Vessel\\LASERTANK\\Tank-go.wav",BOTHVIEW_FADED_FAR);
RequestLoadVesselWave3(JohnSoundID,START,"Sound\\Vessel\\LASERTANK\\Tank-start.wav",BOTHVIEW_FADED_FAR);
RequestLoadVesselWave3(JohnSoundID,CANNON,"Sound\\Vessel\\LASERTANK\\LASER.wav",BOTHVIEW_FADED_FAR);
SoundOptionOnOff3(JohnSoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYHOVERTHRUST,FALSE);
SoundOptionOnOff3(JohnSoundID,PLAYATTITUDETHRUST,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(JohnSoundID,DISPLAYTIMER,FALSE);
}// Constructor
LASERTANK1::LASERTANK1 (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)


{
IGNITION_check = NO_START;


RDOOR_status = RDOOR_UP;
IGNITION = 0;
RDOOR_proc = 0;
LIFT_SPEED =.25;
L_SPEED =.1;
t1 = 1;
i2l = 0.5;
i2r = 0.5;
i2=.5;
TURRET_check = TM_STOP;
TURRET_proc=0;
GUN_proc=0;
blinkstate=0;
GUN_check=0;
laserbeam=0;	
	DefineAnimations ();
	
}


void LASERTANK1::clbkSetClassCaps (FILEHANDLE cfg)
{
	// physical specs
	SetSize (11);
	SetEmptyMass (100.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (0.1);
	SetWingEffectiveness (0.1);
	SetCrossSections (_V(35.24,47.96,14.88));
	SetRotDrag (_V(0.1,0.1,0.1));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(8.87,9.06,3.07 ));
	SetTrimScale (0.05);
	SetCameraOffset (_V(-.589,.83,2.15));
	
	SetTouchdownPoints  (_V(0,-2.58,1), _V(-1.6,-2.58,-1), _V(1.6,-2.58,-1));; 

DOCKHANDLE main1=CreateDock (_V(0,1,0), _V(0,1,0), _V(0,0,-1));
ATTACHMENTHANDLE MAIN;
MAIN = CreateAttachment (true, _V(0,-2.58,0),_V(0,1,0),_V(0,0,1),"BTM",false);	
EnableTransponder (true);
EnableIDS (main1, true);
SetIDSChannel(main1,8);

    PROPELLANT_HANDLE TANK;
    TANK = CreatePropellantResource (FUELMASS);

	// ***************** thruster definitions *******************
	

	th_main = CreateThruster (_V(0,0,-4.3), _V(0,0,1), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
		
	th_retro = CreateThruster (_V(0,0,4.3), _V(0,0,-1), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (&th_retro, 1, THGROUP_RETRO);

	th_rcs[0] = CreateThruster (_V( 1,0, 1), _V(0,0,1),MAXRCSTH, TANK, ISP);
	//left front tread
	th_rcs[1] = CreateThruster (_V(-1,0, 1), _V(0,0,1), MAXRCSTH, TANK, ISP);
	// right front tread
	th_rcs[2] = CreateThruster (_V( 1,0,-1), _V(0,0,-1), MAXRCSTH, TANK, ISP);
	//left rear tread
	th_rcs[3] = CreateThruster (_V(-1,0,-1), _V(0,0,-1), MAXRCSTH, TANK, ISP);
    // right rear tread
	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[2];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	// ************************************************
	// Call clbkSetClassCaps_UMMU and select a member
	SelectedUmmuMember=0;
	clbkSetClassCaps_UMMu();	
	
// visual specs
SetMeshVisibilityMode (AddMesh (oapiLoadMeshGlobal ("LASER1")), MESHVIS_ALWAYS); //Main ship mesh
}
// =======================================================
// UMMU Class Caps
// =======================================================
void LASERTANK1::clbkSetClassCaps_UMMu(void)
{
	// InitUmmu
	UmmuSdKInitReturnCode=Crew.InitUmmu(GetHandle());	
	

	// GetUserUMmuVersion
	float UMmuVersion=Crew.GetUserUMmuVersion();

	// DefineAirLockShape
	Crew.DefineAirLockShape(FALSE,-2,2,-3,0,0,7);			// Airlock open, 2 meter large 4 meter high 6 meter long (vessel local coordinate)

	// SetMembersPositionOrientationOnEVA
	Crew.SetMembersPosRotOnEVA(_V(0,0,6.5),_V(0,0,0));	// 3 meters in front of ship (z) facing direction of ship (vessel local coordinate)

	//maximum seat available in this ship
	Crew.SetMaxSeatAvailableInShip(2);

	//default crew member used when the ship is spawned by Orbiter's scenerio editor. 
	Crew.AddCrewMember("Paul Roper",41,65,74,"Capt");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Frank Craig",27,67,55,"Eng");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Craig Collins",25,70,45,"Ast");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Doug Jackson",45,70,45,"Doc" );	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Keith Ford",45,70,45,"Sec");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Paul Foster",45,70,45,"Sci" );	//(for name and id a-z A-Z 0-9 characters only)
//change mesh
	
	// see function "SendFocusScreenMessage"
	strcpy(cOurMessage,"");
	//change mesh
	//Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");
}

void LASERTANK1::clbkPostStep(double simt, double simdt, double mjd)
{
	//---------------------------------------------------------------------------
	// ProcessUniversalMMu
	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
		case UMMU_TRANSFERED_TO_OUR_SHIP: 
			sprintf(SendFocusScreenMessage(),"%s \"%s\" aged %i was transfered to our ship",Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
			break;
		case UMMU_RETURNED_TO_OUR_SHIP:
			sprintf(SendFocusScreenMessage(),"%s \"%s\" aged %i entered into our ship",Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
			break;
	}

	//----------------------------------------------------------------------------
	// SetCrewMemberPulseBySlotNumber
	if(GroundContact()==TRUE)
	{
		// we check vertical speed
		int I;
		VECTOR3 vHorizonAirspeedVector={0};
		GetHorizonAirspeedVector (vHorizonAirspeedVector);
		double VertSpeed		=vHorizonAirspeedVector.y;
		if(VertSpeed<-3)
		{
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all :(
			for(I=0;I<Crew.GetCrewTotalNumber();I++)
			{
				Crew.SetCrewMemberPulseBySlotNumber(I,0);	// set cardiac pulse to zero
			}
			strcpy(SendFocusScreenMessage(),"Oooh no ! Crash - All crew aboard killed");
		}
	}

	//----------------------------------------------------------------------------
	// see function "SendFocusScreenMessage" below for explaination.
	if(GetHandle()==oapiGetFocusObject())
	{
		strcpy(oapiDebugString(),cOurMessage);
	}

double vsAirSpeed = GetAirspeed();	
i3 = GetControlSurfaceLevel(AIRCTRL_RUDDER);

VECTOR3 speedvec; // first we define our vector which will recieve the data
oapiGetFocusShipAirspeedVector(&speedvec);//then we retrieve its x,y,z

if (GroundContact()==true && GetAirspeed() >13 && (speedvec.z>0))
{AddForce(_V(0,0,-9e3),_V(0,0,0));}

if (GroundContact()==true && GetAirspeed() >13 && (speedvec.z<0))
{AddForce(_V(0,0,9e3),_V(0,0,0));}

double da = simdt * LIFT_SPEED;



//Anim Wheels Forward
 
 if (GetThrusterLevel (th_main) >0 && i3 == 0)
 {
    if (i > 1)i=0;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i > 1)i=0;
    i = i + (vsAirSpeed /300);
     
    if (vsAirSpeed ==0) i=0;
    
  // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
    main_on = 1;
    retro_on = 0;
}

}

//Anim Wheels Backward

if (GetThrusterLevel (th_retro) >0 && i3 == 0)

{
    
    if (i < 0)i=1;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (vsAirSpeed /300);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
     main_on = 0;
    retro_on = 1;
    
}


}


//Anim Wheels Forward and Backward when moving without power

 //Anim Wheels Forward and Backward when moving without power

if (GetThrusterLevel (th_main)  == 0 && GetThrusterLevel (th_retro) == 0 && vsAirSpeed > 0.15 && i3 == 0)

{
    
	
	if (main_on == 0 && retro_on == 1)
	
	{
    if (i < 0)i=1;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (vsAirSpeed /300);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
     
    
}
}

if (main_on == 1 && retro_on == 0)
	
	{
    if (i > 1)i=0;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i >  1)i=0;
    i = i + (vsAirSpeed /300);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
    
    
    
}
}


}




//Anim Wheels Opposite Forward and Backward when turning

// Left turn
if (i3 < -0.15)

{
    
    if (i < 0)i=1;
    
    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (0.01 + (vsAirSpeed /100));
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i2 );
    
	SetAnimation (anim_wheelrightrev, i);
    SetAnimation (anim_wheelleft, i);
    
  
}

// Right turn
if (i3 > 0.15)

{
    
    if (i < 0)i=1;
    
    
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (0.01 + (vsAirSpeed /100));
     
    if (vsAirSpeed ==0) i=1;
    
 
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleftrev, i);
    
   
    
}

    
    //Anim Levers
    //sprintf(oapiDebugString(),"anim %2.2f", ignition );
    // Forward Lever animation
    
    if (GetThrusterLevel (th_main) >0 && i3 == 0)
   //Forward Thrust applied not turning, move levers back 
    {
	    i2=i2 - 0.01;
	    if (i2 < .25) i2=.25;
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
	
    
       
	    if (GetThrusterLevel (th_main) == 0 && i2 < 0.5 && i3 == 0)
    
    {
	    i2=i2 + 0.01;
	    if (i2 > 0.5) i2= 0.5;
	    // no thrust, return levers to neutral
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    
    if (GetThrusterLevel (th_main) == 0 && GetThrusterLevel (th_retro) ==0 && i2 == 0.5 && i3 == 0)
    
    {
	    i2 = 0.5;
	   
	  //no thrust, lever neutral  
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    // Reverse Lever animation
    
    if (GetThrusterLevel (th_retro) >0 && i3 == 0 )
    
    {
	    i2=i2 + 0.01;
	    if (i2 > .75) i2= .75;
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
	
    //retro thrust move lever forward
       
	    if (GetThrusterLevel (th_retro) == 0 && i2 > 0.5 && i3 == 0)
    
    {
	    i2=i2 - 0.01;
	    if (i2 < 0.5) i2= 0.5;
	    
	 //no retro thrust return levers to neutral
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    //Turn Lever animation
    
   
    if ( i3 > 0)
    
    {
	    i2= 0.5;
	    i2l = i2l - 0.05;
	    if (i2l < 0) i2l= 0;
	    
	    i2r = i2r + 0.05;
	    if (i2r > 1) i2r= 1;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    if ( i3 == 0 && i2l < 0.5)
    
    {
	    i2l = i2l + 0.05;
	    if (i2l > 0.5) i2l= 0.5;
	    
	    i2r = i2r - 0.05;
	    if (i2r < 0.5) i2r= 0.5;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    
    if ( i3 < 0)
    
    {
	    i2= 0.5;
	    i2l = i2l + 0.05;
	    if (i2l > 1) i2l= 1;
	    
	    i2r = i2r - 0.05;
	    if (i2r < 0) i2r= 0;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    if ( i3 == 0 && i2l > 0.5)
    
    {
	    i2l = i2l - 0.05;
	    if (i2l < 0.5) i2l= 0.5;
	    
	    i2r = i2r + 0.05;
	    if (i2r > 0.5) i2r= 0.5;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
 
      
	
if (RDOOR_status >= RDOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (RDOOR_status == RDOOR_RAISING) {
			if (RDOOR_proc > 0.0) RDOOR_proc = max (0.0, RDOOR_proc-da);
			else                RDOOR_status = RDOOR_UP;
		} else {
			if (RDOOR_proc < 1.0) RDOOR_proc = min (1.0, RDOOR_proc+da);
			else                RDOOR_status = RDOOR_DOWN;
		}
SetAnimation (anim_reardoor, RDOOR_proc);




}
{
 //turrent

double db = simdt * LIFT_SPEED;

    if (TURRET_proc > 1) TURRET_proc = (TURRET_proc - 1);
    else if (TURRET_proc < 0) TURRET_proc = (TURRET_proc + 1);

   if (TURRET_check == TM_LEFT)TURRET_proc = (TURRET_proc + db);
    else if (TURRET_check == TM_RIGHT)TURRET_proc = (TURRET_proc - db);
    else if (TURRET_check == TM_STOP)TURRET_proc = TURRET_proc;

    SetAnimation (anim_TURRET, TURRET_proc);


}	
{
	    
  //GUN

double dc = simdt * LIFT_SPEED;

    if (GUN_proc > 1) GUN_proc = 1;
    else if (GUN_proc < 0) GUN_proc = 0;

   if (GUN_check == 1)GUN_proc = (GUN_proc + dc);
    else if (GUN_check == 2)GUN_proc = (GUN_proc - dc);
    else if (GUN_check == 0)GUN_proc = GUN_proc;

   SetAnimation (anim_gun, GUN_proc);

}

if (LASER==1){


double lm = LIFT_SPEED* simdt;

if ((laserbeam > 0))
blinkstate=1;
if ((laserbeam > 0.025))
blinkstate =0;
if ((laserbeam > 0.05))
blinkstate=1;
if ((laserbeam > .075))
blinkstate =0;
if ((laserbeam> .1))
blinkstate=1;
if ((laserbeam > .125))
blinkstate =0;
if ((laserbeam> .15))
blinkstate = 1;
if ((laserbeam > .175))
blinkstate =0;
if ((laserbeam> .2))
blinkstate = 1;
if ((laserbeam > .225))
blinkstate =0;
if ((laserbeam > .25))
blinkstate=1;
if ((laserbeam > 0.275))
blinkstate =0;
if ((laserbeam > 0.3))
blinkstate=1;
if ((laserbeam > .325))
blinkstate =0;
if ((laserbeam> .35))
blinkstate=1;
if ((laserbeam > .375))
blinkstate =0;
if ((laserbeam> .4))
blinkstate = 1;


//sprintf(oapiDebugString(),"anim %2.2f", blinkstate);
SetAnimation (anim_beam, blinkstate);
laserbeam=laserbeam+lm;
if (laserbeam>.4){
laserbeam=0;
blinkstate=0;
SetAnimation (anim_beam, blinkstate);
LASER=0;}

}	    	    

// engine sound
double pit = (vsAirSpeed * pit2) + 11025;
double sp = (190 +((vsAirSpeed)*5));
if (sp>=225)sp=225;

if (IGNITION_check == YES_START) {PlayVesselWave3(JohnSoundID,ENG,LOOP,sp,pit);
     
   

// Only turn on the engine sound if it is not already running AND if velocity is >= 10 meters-per-second;
// otherwise, only turn OFF the engine sound if it is running AND if velocity is < 10 meters-per-second.
if (!IsPlaying3(JohnSoundID, ENG) && (vsAirSpeed >= 0))
{
    // turn on the engine sound
    PlayVesselWave3(JohnSoundID,ENG,LOOP,190);
}
else if (IsPlaying3(JohnSoundID, ENG) && (vsAirSpeed < 0))
{
    // turn off the engine sound
    StopVesselWave3(JohnSoundID, ENG);
}

}


// -------------------- added ------------------------------------

if (IGNITION_check == NO_START) {
    
        SetThrusterGroupLevel(th_main,0);
		
		SetThrusterLevel(th_main,0);
		
		SetThrusterMax0 (th_main ,0);
		
		SetThrusterGroupLevel(th_retro,0);
		
		SetThrusterLevel(th_retro,0);
		
		SetThrusterMax0 (th_retro ,0);
		
		
		SetThrusterLevel (th_rcs[0], 0);
		SetThrusterMax0 (th_rcs[0] ,0);
		
        SetThrusterLevel (th_rcs[1], 0);
        SetThrusterMax0 (th_rcs[1] ,0);
        
        SetThrusterLevel (th_rcs[2], 0);
        SetThrusterMax0 (th_rcs[2] ,0);
        
        SetThrusterLevel (th_rcs[3], 0);
        SetThrusterMax0 (th_rcs[3] ,0);
        


	
}

 if (IGNITION_check == YES_START){
		
		
		SetThrusterMax0 (th_main ,9000);
		
		
		SetThrusterMax0 (th_retro ,9000);
		
		SetThrusterMax0 (th_rcs[0] ,90000);
		
		SetThrusterMax0 (th_rcs[1] ,90000);
		
		SetThrusterMax0 (th_rcs[2] ,90000);
		
		SetThrusterMax0 (th_rcs[3] ,90000);

	

//------------------------------------------------------------





}

}






void LASERTANK1::DefineAnimations(void)

{

   	static UINT EGrp20[2] = {15,116};//turret
   	static UINT EGrp21[2] = {11,138};//gun
   	static UINT EGrp22[1] = {146};//beam
   static MGROUP_ROTATE TURRET (0,EGrp20, 2, _V(0,1.976,-.837), _V(0,1,0), (float)(360*RAD));
 
   GUN = new MGROUP_ROTATE(0,EGrp21, 2, _V(0,2.503,-.837), _V(1,0,0), (float)(-50*RAD));
   BEAM = new MGROUP_TRANSLATE(0,EGrp22, 1, _V(0,0,2));
ANIMATIONCOMPONENT_HANDLE parent, parent2;
    
anim_TURRET = CreateAnimation (0.0);
parent = AddAnimationComponent (anim_TURRET, 0, 1, &TURRET);
anim_gun = CreateAnimation (0.0);

anim_beam = CreateAnimation (0.0);
parent2 = AddAnimationComponent (anim_gun, 0, 1, GUN, parent);
AddAnimationComponent (anim_beam, 0, 1, BEAM, parent2);  



  
  



	
 // Levers
   static UINT LeGrp2[2] = {82,148};
   static MGROUP_ROTATE lever2 (0,LeGrp2, 2, _V(-.412,-.605,2.904), _V(1,0,0), (float)(30*RAD));
   static UINT LeGrp1[2] = {81,147};
   static MGROUP_ROTATE lever1 (0,LeGrp1, 2, _V(-.702,-.605,2.904), _V(1,0,0), (float)(30*RAD));
   
   
      
      
   anim_leverleft = CreateAnimation (0);
      
   AddAnimationComponent  (anim_leverleft, 0, 1, &lever1);
   
   anim_leverright = CreateAnimation (0);
   
   AddAnimationComponent  (anim_leverright, 0, 1, &lever2);   
   
   anim_leverleft1 = CreateAnimation (0);
      

   	
	
//CANOPY

   	static UINT EGrp17[28] = {153,152};
   
   static MGROUP_ROTATE rdoor (0,EGrp17, 2, _V(0,.911,1.248), _V(1,0,0), (float)(-40*RAD));
   
   anim_reardoor = CreateAnimation (0);
   
   AddAnimationComponent  (anim_reardoor, 0, 1, &rdoor);  
    
    
 //wheelLGEAR
 static UINT WGrp1[1] = {95};
   static MGROUP_ROTATE wheel1 (0,WGrp1, 1, _V(-5,-1.382,-3.904), _V(1,0,0), (float)(t1*360*RAD));
   //wheelL1
 static UINT WGrp2[1] = {102};
   static MGROUP_ROTATE wheel2 (0,WGrp2, 1, _V(-5,-1.413,4.531), _V(1,0,0), (float)(t1*360*RAD));
      //wheelL2
 static UINT WGrp3[1] = {108};
   static MGROUP_ROTATE wheel3 (0,WGrp3, 1, _V(-5,-1.965,3.434), _V(1,0,0), (float)(t1*360*RAD));      
 //wheelL3
static UINT WGrp4[1] = {107};
   static MGROUP_ROTATE wheel4 (0,WGrp4, 1, _V(-5,-1.965,2.282), _V(1,0,0), (float)(t1*360*RAD));
    //wheelL4
 static UINT WGrp5[1] = {106};
   static MGROUP_ROTATE wheel5 (0,WGrp5, 1, _V(-5,-1.965,1.02), _V(1,0,0), (float)(t1*360*RAD));
    //wheelL5
 static UINT WGrp6[1] = {105};
   static MGROUP_ROTATE wheel6 (0,WGrp6, 1, _V(-5,-1.965,-.238), _V(1,0,0), (float)(t1*360*RAD));
    //wheelL6
 static UINT WGrp7[1] = {104};
   static MGROUP_ROTATE wheel7 (0,WGrp7, 1, _V(-5,-1.965,-1.476), _V(1,0,0), (float)(t1*360*RAD));
       //wheelL7
 static UINT WGrp8[1] = {103};
   static MGROUP_ROTATE wheel8 (0,WGrp8, 1, _V(-5,-1.967,-2.667), _V(1,0,0), (float)(t1*360*RAD));

  //wheelRGEAR 
   static UINT WGrp9[1] = {93};
   static MGROUP_ROTATE wheel9 (0,WGrp9, 1, _V(5,-1.382,-3.904), _V(1,0,0), (float)(t1*360*RAD));
   //wheelR1
 static UINT WGrp10[1] = {115};
   static MGROUP_ROTATE wheel10 (0,WGrp10, 1, _V(5,-1.413,4.531), _V(1,0,0), (float)(t1*360*RAD));
      //wheelR2
 static UINT WGrp11[1] = {114};
   static MGROUP_ROTATE wheel11 (0,WGrp11, 1, _V(5,-1.965,3.434), _V(1,0,0), (float)(t1*360*RAD));     
 //wheelR3
 static UINT WGrp12[1] = {113};
   static MGROUP_ROTATE wheel12 (0,WGrp12, 1, _V(5,-1.965,2.282), _V(1,0,0), (float)(t1*360*RAD));
    //wheelR4
 static UINT WGrp13[1] = {112};
   static MGROUP_ROTATE wheel13 (0,WGrp13, 1, _V(5,-1.965,1.02), _V(1,0,0), (float)(t1*360*RAD));
    //wheelR5
 static UINT WGrp14[1] = {111};
   static MGROUP_ROTATE wheel14 (0,WGrp14, 1, _V(5,-1.965,-.238), _V(1,0,0), (float)(t1*360*RAD));
    //wheelR6
 static UINT WGrp15[1] = {110};
   static MGROUP_ROTATE wheel15 (0,WGrp15, 1, _V(5,-1.965,-1.476), _V(1,0,0), (float)(t1*360*RAD));
       //wheelR7
 static UINT WGrp16[1] = {109};
   static MGROUP_ROTATE wheel16 (0,WGrp16, 1, _V(5,-1.967,-2.667), _V(1,0,0), (float)(t1*360*RAD));
 
   //GEARL1
   static UINT WGrp17[1] = {95};
   static MGROUP_ROTATE wheel17 (0,WGrp17, 1, _V(5,-1.382,-3.904), _V(1,0,0), (float)(-360*RAD));
     //wheelL1
 static UINT WGrp18[1] = {102};
   static MGROUP_ROTATE wheel18 (0,WGrp18, 1, _V(-5,-1.413,4.531), _V(1,0,0), (float)(-360*RAD));
      //wheelL2
 static UINT WGrp19[1] = {108};
   static MGROUP_ROTATE wheel19 (0,WGrp19, 1, _V(-5,-1.965,3.434), _V(1,0,0), (float)(-360*RAD));      
 //wheelL3
 static UINT WGrp20[1] = {107};
   static MGROUP_ROTATE wheel20 (0,WGrp20, 1, _V(-5,-1.965,2.282), _V(1,0,0), (float)(-360*RAD));
    //wheelL4
 static UINT WGrp21[1] = {106};
   static MGROUP_ROTATE wheel21 (0,WGrp21, 1, _V(-5,-1.965,1.02), _V(1,0,0), (float)(-360*RAD));
    //wheelL5
 static UINT WGrp22[1] = {105};
   static MGROUP_ROTATE wheel22 (0,WGrp22, 1, _V(-5,-1.965,-.238), _V(1,0,0), (float)(-360*RAD));
    //wheelL6
 static UINT WGrp23[1] = {104};
   static MGROUP_ROTATE wheel23 (0,WGrp23, 1, _V(-5,-1.965,-1.476), _V(1,0,0), (float)(-360*RAD));
       //wheelL7
 static UINT WGrp24[1] = {103};
   static MGROUP_ROTATE wheel24 (0,WGrp24, 1, _V(-5,-1.967,-2.667), _V(1,0,0), (float)(-360*RAD));
   
   //GEARRIGHT
   static UINT WGrp25[1] = {93};
   static MGROUP_ROTATE wheel25 (0,WGrp25, 1, _V(-5,-1.382,-3.904), _V(1,0,0), (float)(-360*RAD));
       //wheelR1
 static UINT WGrp26[1] = {115};
   static MGROUP_ROTATE wheel26 (0,WGrp26, 1, _V(-5,-1.413,4.531), _V(1,0,0), (float)(-360*RAD));
      //wheelR2
 static UINT WGrp27[1] = {114};
   static MGROUP_ROTATE wheel27 (0,WGrp27, 1, _V(-5,-1.965,3.434), _V(1,0,0), (float)(-360*RAD));     
 //wheelR3
 static UINT WGrp28[1] = {113};
   static MGROUP_ROTATE wheel28 (0,WGrp28, 1, _V(-5,-1.965,2.282), _V(1,0,0), (float)(-360*RAD));
    //wheelR4
 static UINT WGrp29[1] = {112};
   static MGROUP_ROTATE wheel29 (0,WGrp29, 1, _V(-5,-1.965,1.02), _V(1,0,0), (float)(-360*RAD));
    //wheelR5
 static UINT WGrp30[1] = {111};
   static MGROUP_ROTATE wheel30 (0,WGrp30, 1, _V(-5,-1.965,-.238), _V(1,0,0), (float)(-360*RAD));
    //wheelR6
 static UINT WGrp31[1] = {110};
   static MGROUP_ROTATE wheel31 (0,WGrp31, 1, _V(-5,-1.965,-1.476), _V(1,0,0), (float)(-360*RAD));
       //wheelR7
 static UINT WGrp32[1] = {109};
   static MGROUP_ROTATE wheel32 (0,WGrp32, 1, _V(-5,-1.967,-2.667), _V(1,0,0), (float)(-360*RAD)); 
 
   
    anim_wheelright = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel9);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel10);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel11);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel12);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel13);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel14);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel15);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel16);
 anim_wheelleft = CreateAnimation (0);
    
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel3);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel4);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel5);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel6);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel7);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel8);
   
   anim_wheelrightrev = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel25);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel26);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel27);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel28);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel29);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel30);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel31);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel32);
   
   
   anim_wheelleftrev = CreateAnimation (0);
    
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel17);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel18);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel19);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel20);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel21);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel22);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel23);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel24);
  
   
         
  
   }

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
return new LASERTANK1 (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel)
{
if (vessel) delete (LASERTANK1*)vessel;
}

// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int LASERTANK1::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	// only process keydown events
	if (!down) 
		return 0; 

	//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	if(key==OAPI_KEY_E)
	{
		// PERFORM THE EVA
		
		int Returned=EvaCrewMemberWithStandardSpacesuit(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));
		
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(SendFocusScreenMessage(),"Transfer to docked ship Ok - %s transfered",Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(SendFocusScreenMessage(),"EVA OK - %s left the ship",Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy(SendFocusScreenMessage(),"Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(SendFocusScreenMessage(),"Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(SendFocusScreenMessage(),"Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy(SendFocusScreenMessage(),"Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(SendFocusScreenMessage(),"Error, no crew by this name in ship");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(SendFocusScreenMessage(),"Error, docked ship do not use UMmu 1.5, ask author to add it See http://orbiter.dansteph.com");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(SendFocusScreenMessage(),"Misc error with UMMU See http://orbiter.dansteph.com to install it again");
			break;
		}
		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	if(key==OAPI_KEY_1)
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendFocusScreenMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}

		// we test that we select existing member
		if(SelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendFocusScreenMessage(),"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA \"ESC\" to eject",SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	if(key==OAPI_KEY_2)
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendFocusScreenMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}
		if(SelectedUmmuMember>0)
			SelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendFocusScreenMessage(),"Slot %i %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA \"ESC\" to eject",SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}


	//---------------------------------------------------------------------------
	// Ummu Key "A" Switch the airlock door on/off
	if(key==OAPI_KEY_A)
	{RevertRDOOR();
		// switch state
		Crew.SetAirlockDoorState(!Crew.GetAirlockDoorState());
		// display state
		if(Crew.GetAirlockDoorState()==TRUE)
			strcpy(SendFocusScreenMessage(),"Airlock is now open");	
		else
			strcpy(SendFocusScreenMessage(),"Airlock is now closed");	
		return 1;
	}

	//---------------------------------------------------------------------------
	// Eject the guy
	if(key==OAPI_KEY_ESCAPE)
	{
		if(Crew.EjectCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember))==EVA_OK)
			sprintf(SendFocusScreenMessage(),"%s EJECTED",Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
		return 1;
	}

	//---------------------------------------------------------------------------
	// Use a different Mesh (type "C" then EVA someone)
	if(key==OAPI_KEY_C)
	{
		Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");	// the stock mmu of orbiter located in "meshes/mmu.msh"
		strcpy(SendFocusScreenMessage(),"Mesh changed");
		return 1;
	}

	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if(key==OAPI_KEY_S)
	{
		sprintf(SendFocusScreenMessage(),"%i souls aboard ship %s, %i seats available",Crew.GetCrewTotalNumber(),GetName(),2-Crew.GetCrewTotalNumber());
		return 1;
	}

            
if(key==OAPI_KEY_7) // rotate+
{
     if (TURRET_check == TM_STOP) TURRET_check = TM_RIGHT;
        else if (TURRET_check == TM_RIGHT) TURRET_check = TM_STOP;
        else if (TURRET_check == TM_LEFT) TURRET_check = TM_RIGHT;
        return 1;
}    
if(key==OAPI_KEY_8) // rotate-
{
      if (TURRET_check == TM_STOP) TURRET_check = TM_LEFT;
        else if (TURRET_check == TM_LEFT) TURRET_check = TM_STOP;
        else if (TURRET_check == TM_RIGHT) TURRET_check = TM_LEFT;
        return 1;
}
		
if(key==OAPI_KEY_5) // rotate+
{
     if (GUN_check == 0) GUN_check = 1;
        else if (GUN_check == 1) GUN_check = 0;
        else if (GUN_check == 2) GUN_check = 1;
        return 1;
}   
if(key==OAPI_KEY_6) // rotate-
{
      if (GUN_check == 0) GUN_check = 2;
        else if (GUN_check == 2) GUN_check = 0;
        else if (GUN_check == 1) GUN_check = 2;
        return 1;	
}
if(key==OAPI_KEY_K)
{
          if(IGNITION_check == NO_START)// not started
			{PlayVesselWave3(JohnSoundID,START,NOLOOP,255,22100);//play start wav
			IGNITION_check=YES_START; }// started
			
		
      return 1;
}
	if(key==OAPI_KEY_W)// fire
	{
LASER=1;
PlayVesselWave3(JohnSoundID,CANNON,NOLOOP,255);      
return 1;
	}

		
		return 0;
}

// ====================================================================
// clbkVisualCreated used to display UMMU initialisation message 
// because oapiDebugString() doesn't work in clbkSetClassCap
// ====================================================================
void LASERTANK1::clbkVisualCreated (VISHANDLE vis, int refcount)
{   
	MainExternalMeshVisual = GetMesh(vis,0);
	// We warn the user, in the case UMmu isn't installed it's a good idea to send a "oapiDebugString"
	// Orbiter message, otherwise your addon will not be fully operational and user may not notice. 
	//(nobody read doc, you may be the only one, if you read this send me a postcard ;)
	if(UmmuSdKInitReturnCode==-999)
		strcpy(SendFocusScreenMessage(),"This addon require UMMU addon wich is not installed or outdated. See http://orbiter.dansteph.com to install it");
	else if(UmmuSdKInitReturnCode!=1)
		strcpy(SendFocusScreenMessage(),"Misc error with UMMU See http://orbiter.dansteph.com to install it again");
	// just for the demo
	else if(UmmuSdKInitReturnCode==1)
		strcpy(SendFocusScreenMessage(),"Welcome aboard. UMMU Keys:  E=EVA 1=&2=select member A=close/open airlock ESC=eject S=get ship info");
}
// ==============================================================
// Visual destroyed
// ==============================================================
void LASERTANK1::clbkVisualDestroyed (VISHANDLE vis, int refcount)
{
	MainExternalMeshVisual = 0;	
}


void LASERTANK1::RevertRDOOR (void)
{
	RDOOR_status = ((RDOOR_status == RDOOR_UP || RDOOR_status == RDOOR_RAISING) ?
		RDOOR_LOWERING : RDOOR_RAISING);
}






// ========================================================================
// clbkLoadState function of Orbiter - Save scenario when user exit Orbiter 
// ========================================================================
void LASERTANK1::clbkSaveState(FILEHANDLE scn)
{

	// ORBITER, default vessel parameters
	SaveDefaultState (scn);

	// UMmu, this will save all our members of crew in scenario
	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu's airlock state.
	Crew.SaveAllMembersInOrbiterScenarios(scn);
}
// ========================================================================
// clbkLoadStateEx function of Orbiter - Load Orbiter scenario 
// ========================================================================
void LASERTANK1::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{

		// UMMU, Load all saved member from scenario. 
		// Return TRUE when a "UMMUCREW" line is encountered 
		// FALSE when it must pass this line to you or default parser
		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE)
			continue;

		// ORBITER, unrecognised option - pass to Orbiter's generic parser
		ParseScenarioLineEx (line, status);
	}

}
//------------------------------------------------------------------------------------------------------------------
// This below is not mandatory for UMmu to run.
//------------------------------------------------------------------------------------------------------------------

// ==========================================================
// USING TEXT INTERFACE FOR CREW MANAGMENT (not a good idea)
//
// This is the custom functions that send oapiDebugString() Message on screen
// it simply avoid that the message is sent if our ship don't have the focus
// and ensure that message will be reminded when we focus back on ship
// the Orbiter function below clbkFocusChanged is used to erase message when
// we lost focus.
// It's a bad idea to do an UMmu user interface only by using Debug screen message 
// but if you can't do better you may use this system.
///////////////////////////////////////////////////////////////////////////////////////////
char *LASERTANK1::SendFocusScreenMessage()
{
	return cOurMessage;
}
void LASERTANK1::clbkFocusChanged(bool getfocus,OBJHANDLE hNewVessel,OBJHANDLE hOldVessel)
{
	// erase message if we focus to another vessel
	if(!getfocus)
	{
		strcpy(oapiDebugString(),"");
	}
}
// ================================================================
// HUD (I put the anim state on the hud because in Poststep
// it forbids ummu messages to be displayed).
// ================================================================


// ==========================================================================================
// USING STANDARD FUNCTION ID,NAME AND SPACESUIT (Good idea)
// **TEST: replace "Crew.EvaCrewMember" with this function in clbkConsumeBufferedKey (key E)**
// ==========================================================================================
int LASERTANK1::EvaCrewMemberWithStandardSpacesuit(char* CrewName)
{
	char TempMiscID[5]={0};
	int I=0;
	char MeshToUse[26]={"ummu/ummu"}; // default mesh if not found
	// retriew the crew's MiscID
	strcpy(TempMiscID,Crew.GetCrewMiscIdByName(CrewName));
	// search if we find this ID in the list
	for(I=0;I<NUMBER_OF_MMU_TYPE;I++)
	{
		if(strcmp(UmmuMiscID[I],TempMiscID)==0)
		{
			// found ! copy the mesh filename to use for EVA
			strcpy(MeshToUse,UmmuMeshUsed[I]);
			break;
		}
	}
	// Set the mesh filename to use
	Crew.SetAlternateMeshToUseForEVASpacesuit(MeshToUse);
	// Eva the crew member with this mesh
	return Crew.EvaCrewMember(CrewName);
}
