// ==============================================================
//                 ORBITER MODULE: LASERTANK1
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// LASERTANK1.cpp
// Control module for LASERTANK1 vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================



#include "orbitersdk.h"
#include "UMmuSDK.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double FUELMASS = 300;
const double ISP = 11e12;
const double MAXMAINTH = 9000;
const double MAXRETROTH = 9000;
const double MAXRCSTH = 90000;

// Interface for derived vessel class: QJBStar
// ==========================================================

class LASERTANK1: public VESSEL2 {
public:
    LASERTANK1 (OBJHANDLE hObj, int fmodel);
	void clbkSaveState(FILEHANDLE scn);
       void clbkPostCreation (void);
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
 	void DefineAnimations(void); 
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkPostStep (double simtt, double simdt, double mjd);
	void clbkFocusChanged(bool getfocus,OBJHANDLE hNewVessel,OBJHANDLE hOldVessel);
enum RDOORStatus { RDOOR_UP, RDOOR_DOWN, RDOOR_RAISING, RDOOR_LOWERING } RDOOR_status;void RevertAIRLOCKOUTER (void);
    void RevertRDOOR (void);
    void Revertlaser (void);
	
// UMMU 1.5 DECLARATION
	UMMUCREWMANAGMENT Crew;
	void clbkSetClassCaps_UMMu(void);	// our special SetClassCapfunction added for more readability
	int SelectedUmmuMember;				// for the SDK demo, select the member to eva
	int UmmuSdKInitReturnCode;			// to display init message to user (in case Ummu isn't installed on Orbiter)

	// our custom oapiDebugString() function (an orbiter function that display debug text at bottom of screen)
	// Our custom function display only message if we have focus. The variables below is used by this function.
	// don't forget if you use such text interface in your vessel that you must initialise the variables
	// in set class cap and do also some code in clbkFocusChanged
	char cOurMessage[1000];
	char *SendFocusScreenMessage();	

	// If you want to use standard ID function see bottom of UMmuExample.cpp
	int EvaCrewMemberWithStandardSpacesuit(char* CrewName);	
private:
////////////////////////Animation example Code//////////////////////////////
////for those "new" Mgroup_Transforms, you need some defintions here in the private section
	
MGROUP_TRANSFORM *TURRET, *GUN, *BEAM;


///////////////////////End Animation Sample Code///////////////////////////////////////


enum TURRET_check { TM_LEFT, TM_STOP, TM_RIGHT } TURRET_check;
THRUSTER_HANDLE th_main, th_retro,  th_rcs[4], th_group[2] ;
enum IGNITION_check { YES_START, NO_START } IGNITION_check;
UINT anim_TURRET;
UINT anim_gun;
UINT anim_beam;
UINT anim_radar;
UINT anim_reardoor;
UINT anim_reardoor1;
UINT anim_wheel;
UINT anim_wheelleft;
UINT anim_wheelright;
UINT anim_wheelleftrev;
UINT anim_wheelrightrev;
UINT anim_leverleft;
UINT anim_leverright;
UINT anim_leverleft1;
UINT anim_leverright1;
UINT anim_dhandle2;
UINT anim_dhandle1;
int JohnSoundID;
double RDOOR_proc;
double LIFT_SPEED;
double L_SPEED;
double ANT_OPERATING_SPEED;
double ant_proc;
double db;
double LASER;
double ant_check;
double blinkstate;
double laserbeam;
double i;
double i3;
double t1;
double main_on;
double retro_on;
double i2l;
double i2r;
double i2;
double pit;
double pit2;
double TURRET_proc;
double GUN_proc;
double GUN_check;
double IGNITION;
};